#include "ElInfo.h"

namespace AMDiS {

  template<typename T>
  T* OperatorTerm::getVectorAtQPs(DOFVectorBase<T>* vec,
				  const ElInfo* elInfo, 
				  SubAssembler* subAssembler,
				  Quadrature *quad)
  {
    FUNCNAME("OperatorTerm::getVectorAtQPs()");

    TEST_EXIT_DBG(elInfo->getMesh() == vec->getFeSpace()->getMesh())
      ("There is something wrong!\n");
      
    return subAssembler->getVectorAtQPs(vec, elInfo, quad);
  }


  template<typename T>
  T* OperatorTerm::getVectorAtQPs(DOFVectorBase<T>* vec,
				  const ElInfo* smallElInfo, 
				  const ElInfo* largeElInfo, 
				  SubAssembler* subAssembler,
				  Quadrature *quad)
  {
    FUNCNAME("OperatorTerm::getVectorAtQPs()");

    TEST_EXIT(smallElInfo->getMesh() == vec->getFeSpace()->getMesh() ||
	      largeElInfo->getMesh() == vec->getFeSpace()->getMesh())
      ("There is something wrong!\n");

    if (smallElInfo->getLevel() == largeElInfo->getLevel()) {

      // Both elements have the same size, so we can use the simple procedure
      // to determine the vecAtQPs.
      
      if (vec->getFeSpace()->getMesh() == smallElInfo->getMesh())
	return subAssembler->getVectorAtQPs(vec, smallElInfo, quad);
      else
	return subAssembler->getVectorAtQPs(vec, largeElInfo, quad);      

    } else {

      // The two elements are different. If the vector is defined on the mesh of the
      // small element, we can still use the simple procedure to determine the vecAtQPs.

      if (vec->getFeSpace()->getMesh() == largeElInfo->getMesh())
	return subAssembler->getVectorAtQPs(vec, smallElInfo, largeElInfo, quad);
      else
	return subAssembler->getVectorAtQPs(vec, smallElInfo, quad);
    }
  }

}
